import { AfterViewInit, Component, Input, OnInit, ViewChild } from '@angular/core';
import { NgxMoveableComponent, OnDragEnd, OnResize, OnResizeEnd } from 'ngx-moveable';
import { OnDrag } from 'ngx-moveable';
import { ConfigService } from '../services/config.service';
import { Rect } from '../models/generated/Rect';
import { Plan } from '../models/plan';
import { Color } from '../color-picker/color-picker.component';

@Component({
  selector: 'plan-display',
  templateUrl: './plan-display.component.html',
  styleUrls: ['./plan-display.component.scss']
})
export class PlanDisplayComponent implements OnInit, AfterViewInit {
  draggable: boolean = true;
  initialized: boolean = false;

  @Input()
  plan: boolean = false;

  @Input()
  backgroundColor?: Color;

  @ViewChild("moveableRef")
  moveableRef!: NgxMoveableComponent;

  constructor(private configService: ConfigService) {}

  ngOnInit(): void {
    
  }

  ngAfterViewInit(): void {
    let init = this.configService.config.initialPlanWindowPosition;
    this.setPosition(init.x, init.y);
    this.setArea(init.width, init.height);
    this.initialized = true;
    console.log("init: ", init);
    console.log("cfg at init: ", this.configService.config);
  }

  setPosition(x: number, y: number) {
    this.moveableRef.request("draggable", {
      x,
      y,
    }, true);
  }

  setArea(width: number, height: number) {
    this.moveableRef.request("resizable", {
      offsetWidth: width,
      offsetHeight: height,
    }, true);
  }

  toRect(): Rect {
    let rectInfo = this.moveableRef.getRect();
    return {
      x: rectInfo.left,
      y: rectInfo.top,
      width: rectInfo.width,
      height: rectInfo.height
    };
  }

  onDrag(e: OnDrag) {
    e.target.style.transform = e.transform;
  }

  onDragEnd(e: OnDragEnd) {
    if (this.initialized) {
      this.configService.config.initialPlanWindowPosition = this.toRect();
    }
  }

  onResize(e: OnResize) {
    e.target.style.width = `${e.width}px`;
    e.target.style.height = `${e.height}px`;
    e.target.style.transform = e.drag.transform;
  }

  onResizeEnd(e: OnResizeEnd) {
    if (this.initialized) {
      this.configService.config.initialPlanWindowPosition = this.toRect();
    }
  }
}
