import { Injectable } from '@angular/core';
import { ShortcutService } from './shortcut.service';
import { EventsService } from './events.service';
import { Event } from "@tauri-apps/api/event";
import { invoke } from '@tauri-apps/api';
import { ConfigService } from './config.service';

class StateEvent {
  Visible?: any;
  Interactable?: any;
  Hidden?: any;
}

@Injectable({
  providedIn: 'root'
})
export class OverlayService {
  interactable: boolean = false;
  isBinding: boolean = false;


  constructor(private shortcuts: ShortcutService, private events: EventsService, private configService: ConfigService) {
    console.log("cfg", configService);
    this.shortcuts.register(this.configService.config.toggleOverlay, this.onToggleOverlay);
    this.events.listen<StateEvent>("OverlayStateChange").subscribe(this.onOverlayStateChange);
  }

  onOverlayStateChange(event: Event<StateEvent>) {
    this.interactable = event.payload.Interactable != null;
  }

  onToggleOverlay = () => {
    if (this.interactable) {
      this.interactable = false;
    } else {
      this.interactable = true;
    }

    invoke("set_interactable", { interactable: this.interactable }).then();
  }

  onBindToggleOverlayFinish(keys: string[]) {
    this.isBinding = false;
    let chord = keys.reduce((acc, curr) => {
      if (acc === '') return curr;

      return acc.concat('+').concat(curr);
    }, '');
    console.log(chord);

    this.shortcuts.rebind(chord, this.onToggleOverlay);
  }
}
